--[[############################################################################################]]--
--[[                                                                                            ]]--
--[[  File     :  /maps/Solo ACU/Solo ACU_script.lua                                            ]]--
--[[                                                                                            ]]--
--[[  Author(s):  Manimal (scripter) + Saya (help) + ertwyu, BennyMoore, KrimsonStorm (concept) ]]--
--[[                                                                                            ]]--
--[[  Version  :  5                                                                             ]]--
--[[                                                                                            ]]--
--[[  Date     :  10th/10/2007                                                                  ]]--
--[[                                                                                            ]]--
--[[  Summary  :  Solo ACU Map Script (1.1.3269 compatible)                                     ]]--
--[[                                                                                            ]]--
--[[############################################################################################]]--


--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Local Definitions                                                         ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua')
local ScenarioFramework = import('/lua/ScenarioFramework.lua')
local Utilities = import('/lua/Utilities.lua')
local UIUtil = import('/lua/ui/uiutil.lua')




--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Misc Global Definitions                                                   ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


ScenarioInfo.Attacker = 1
ScenarioInfo.Defender = 2

theAttacker           = ScenarioInfo.Attacker
theDefender           = ScenarioInfo.Defender

T1PDunicBTmult        = 1.2




--[[                                                                             ]]--
--[[   ubcsTable                                                                 ]]--
--[[                                                                             ]]--
--[[   Do NOT erase any line in this ubcsTable !                                 ]]--
--[[   Add a comment sign instead for an evenutal future use                     ]]--
--[[                                                                             ]]--

ubcsTable = {

    { name                  = 'ATTACKER',
      nickname              = ' - ATTACKER',
      army                  = 'ATTACKER',
      enemi                 = 'DEFENDER',
      unitId                = 'url0001', 
      isReady               = false,
      EnhancementsList      = { 'AdvancedEngineering',
                                'CoolingUpgrade',
                                'MicrowaveLaserGenerator',
                                'ResourceAllocation',
                                'T3Engineering'
                              },
      EnhRestrictionsList   = { 'NaniteTorpedoTube',
                                'StealthGenerator'
                              },
      BuildRestrictionsList = { categories.AEON,
                                categories.UEF,
                                categories.NAVAL,
                                categories.EXPERIMENTAL,
                                categories.ura0102,  # Interceptor
                                categories.ura0103,  # Attack Bomber
                                categories.ura0203,  # Gunship
                                categories.ura0204,  # Torpedo Bomber
                                categories.dra0202,  # Bomber Corsair
                                categories.ura0303,  # Air Superiority Fighter
                                categories.ura0304,  # Strategic Bomber
                                categories.urb0304,  # Quantum Gateway
                                categories.urb1102,  # Hydrocarbon Power Plant
                                categories.urb2108,  # Tactical Missile Launcher
                                categories.urb2109,  # Tech 1 Torpedo Launcher
                                categories.urb2205,  # Tech 2 Torpedo Launcher
                                categories.urb2301,  # Tech 2 Point Defense (Heavy Gun Tower)
                                categories.urb2302,  # Long Range Artillery
                                categories.urb2303,  # Light Artillery
                                categories.urb2305,  # Strategic Missile Launcher
                                categories.urb3102,  # Tech 1 Sonar System
                                categories.urb3104,  # Omni Sensor Array
                                categories.urb3202,  # Tech 2 Sonar System
                                categories.urb4203,  # Stealth Field Generator
                                categories.urb4302,  # Strategic Missile Defense
                                categories.urb5101,  # Wall Section
                                categories.urb5103,  # Quantum Gate Beacon
                                categories.url0103,  # Mobile Light Artillery
                                categories.url0106,  # Light Assault Bot
                                categories.url0107,  # Heavy Assault Bot
                                categories.url0111,  # Mobile Missile Launcher
                                categories.url0202,  # Heavy Tank
                                categories.url0203,  # Amphibious Tank
                                categories.drl0204,  # Cybran Rocket Bot
                                categories.url0301,  # Support Command Unit
                                categories.url0303,  # Siege Assault Bot
                                categories.url0304,  # Mobile Heavy Artillery
                                categories.url0306,  # Mobile Stealth Field System
                                categories.url5103,  # Quantum Gate Beacon
                                categories.urs0305   # Sonar Platform
                              },
      BuildTimeMultiplier   = 1
    },

    { name                  = 'DEFENDER',
      nickname              = ' - DEFENDER',
      army                  = 'DEFENDER',
      enemi                 = 'ATTACKER',
      unitId                = 'uel0001', 
      isReady               = false,
      EnhancementsList      = { 'AdvancedEngineering'
                              },
      EnhRestrictionsList   = { 'DamageStablization',
                                'HeavyAntiMatterCannon',
                                'T3Engineering',
                                'TacticalNukeMissile'
                              },
      BuildRestrictionsList = { categories.AEON,
                                categories.CYBRAN,
                                categories.NAVAL,
                                categories.TECH3,
                                categories.EXPERIMENTAL,
                                categories.ueb1102,  # Hydrocarbon Power Plant
                                categories.ueb1201,  # Tech 2 Power Plant
                                categories.ueb2109,  # Tech 1 Torpedo Launcher
                                categories.ueb2205,  # Tech 2 Torpedo Launcher
                                categories.ueb2301,  # Tech 2 Point Defense (Heavy Gun Tower)
                                categories.ueb2302,  # Long Range Artillery
                                categories.ueb2303,  # Light Artillery
                                categories.ueb3102,  # Tech 1 Sonar System
                                categories.ueb3202,  # Tech 2 Sonar System
                                categories.ueb5101,  # Wall Section
                                categories.uel0307   # Mobile Shield Generator
                              },
      BuildTimeMultiplier   = 1
    }

}

# end of ubcsTable


--[[  Table of Units to be spawned                           ]]--
--[[  same format as ubcsTable so these are concatenable     ]]--

unitsTable = {

    { name                  = 'Air_Transport_1',
      nickname              = 'Air Transport - ATTACKER',
      army                  = ubcsTable[theAttacker].army,
      enemi                 = ubcsTable[theDefender].army,
      unitId                = 'ura0104',
      isReady               = false,
      EnhancementsList      = { },
      EnhRestrictionsList   = { },
      BuildRestrictionsList = { },
      BuildTimeMultiplier   = 1
    },

    { name                  = 'Air_Spies_1', 
      nickname              = 'Air Spies - ATTACKER',
      army                  = ubcsTable[theAttacker].army,
      enemi                 = ubcsTable[theDefender].army,
      unitId                = 'ura0302',
      isReady               = false,
      EnhancementsList      = { },
      EnhRestrictionsList   = { },
      BuildRestrictionsList = { },
      BuildTimeMultiplier   = 1
    },

    { name                  = 'Point_Defense_T1_CYB', 
      nickname              = 'T1 P.D. - ATTACKER',
      army                  = ubcsTable[theAttacker].army,
      enemi                 = ubcsTable[theDefender].army,
      unitId                = 'urb2101',
      isReady               = false,
      EnhancementsList      = { },
      EnhRestrictionsList   = { },
      BuildRestrictionsList = { },
      BuildTimeMultiplier   = T1PDunicBTmult
    },

    { name                  = 'Point_Defense_T1_UEF', 
      nickname              = 'T1 P.D. - DEFENDER',
      army                  = ubcsTable[theDefender].army,
      enemi                 = ubcsTable[theAttacker].army,
      unitId                = 'ueb2101',
      isReady               = false,
      EnhancementsList      = { },
      EnhRestrictionsList   = { },
      BuildRestrictionsList = { },
      BuildTimeMultiplier   = T1PDunicBTmult
    },

    { name                  = 'Tactical_Missiles_1', 
      nickname              = 'Tac M.L. - DEFENDER',
      army                  = ubcsTable[theDefender].army,
      enemi                 = ubcsTable[theAttacker].army,
      unitId                = 'ueb2108',
      isReady               = false,
      EnhancementsList      = { },
      EnhRestrictionsList   = { },
      BuildRestrictionsList = { },
      BuildTimeMultiplier   = 1
    }

}



--[[  Table of alive Units including the ones with a tweak  ]]--
--[[   examples:                                            ]]--
--[[     unit.numbIdStr = "1048611"                         ]]--
--[[     unit.nameIdStr = "ueb2101"                         ]]--
--[[     unit.army      = theAttacker                       ]]--

--[[
aliveUnits = {
    { 
      numbIdStr = "1048611",
      nameIdStr = "urb2101",
      army = theAttacker
    },
    { 
      numbIdStr = "1047504",
      nameIdStr = "ueb2101",
      army = theDefender
    },            
}
]]--

aliveUnits = {  }


--[[  Units's BuildTimeTable             ]]--

BuildTimeTable = {
    AeoT1PD = {             # Aeon's Tech 1 Point Defense
        idStr = "uab2101",
        Original  = 250,
        Current   = 250
    },
    UefT1PD = {             # UEF's Tech 1 Point Defense
        idStr = "ueb2101",
        Original  = 250,
        Current   = 250
    },
    CybT1PD = {             # Cybran's Tech 1 Point Defense
        idStr = "urb2101",
        Original  = 250,
        Current   = 250
    }
}


--[[  Miscellaneous Variables and Flags  ]]--

ACUisReady            = false

attackUBC             = false                  # ACU unit for Attacker
defensUBC             = false                  # ACU unit for Defender

PreBuiltBase          = false

A_Air_Transport_1     = false                  # Group of Air Transports

A_Air_Spies_1         = {                      # Group of Air Spies
                          Air_Spy_01 = false,
                          Air_Spy_02 = false,
                          Air_Spy_03 = false,
                          Air_Spy_04 = false,
                          Air_Spy_05 = false,
                          Air_Spy_06 = false,
                          Air_Spy_07 = false,
                          Air_Spy_08 = false,
                          Air_Spy_09 = false,
                          Air_Spy_10 = false
                        }

D_Tactical_Missiles_1 = false                 # Group of Tactical Missiles Launchers





--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Local Tuning Variables                                                    ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--



--[[  The players's unit cap  ]]--
local AttackerUnitCap = 500
local DefenderUnitCap = 500

local tblResult = {}

local mapSizeX, mapSizeY = GetMapSize()





--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Misc Local Definitions                                                    ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--








--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--
--[[                                                                             ]]--
--[[   STARTER FUNCTIONS                                                         ]]--
--[[                                                                             ]]--
--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--





--[[                                                                             ]]--
--[[   OnPopulate()                                                              ]]--
--[[                                                                             ]]--
--[[   alliances and unit spawning go in there so you dont get conflicts and     ]]--
--[[   weird behaviours with initializearmies                                    ]]--
--[[                                                                             ]]--

function OnPopulate()

    LOG(" ")
    LOG("MANIMAL:  Game Engine : ".._VERSION.." .")
    LOG(" ")
    LOG("MANIMAL:  Map Size :  X = "..mapSizeX..", Y = "..mapSizeY.." .")
    LOG(" ")
    LOG("=======>  OnPopulate  ===============================================")
    LOG(" ")



    --[[  GENERAL SETUP                                                          ]]--
    --[[  Scores, UnitCap, Alliances, etc                                        ]]--

    --[[  Fix Scenario's Options  ]]--

    ScenarioInfo.Options.TeamSpawn        = 'fixed'
    ScenarioInfo.Options.TeamLock         = 'locked'
    ScenarioInfo.Options.FogOfWar         = 'explored' 
    ScenarioInfo.Options.CivilianAlliance = 'neutral' 
    ScenarioInfo.Options.CheatsEnabled    = 'false' 
    ScenarioInfo.Options.Victory          = 'demoralization'  ## WHY NOT 'assassination' ?!? ;
    ScenarioInfo.Options.UnitCap          = '500' 
    ScenarioInfo.Options.NoRushOption     = 'Off' 
    ScenarioInfo.Options.Timeouts         = '-1'
    ScenarioInfo.Options.PrebuiltUnits    = 'Off' 
    ScenarioInfo.Options.GameSpeed        = 'fixed'


    --[[  Fix players's Unit Cap, NO EXTRA UNITS ALLOWED  ]]--

    SetIgnoreArmyUnitCap( theAttacker, false )
    SetIgnoreArmyUnitCap( theDefender, false )

    --[[  Set ALL Players's ALLIANCES  ]]--

    ArmyBrains[theAttacker].RequestingAlliedVictory = false
    ArmyBrains[theDefender].RequestingAlliedVictory = false
    SetAlliance( ubcsTable[theAttacker].name, ubcsTable[theDefender].name, 'Enemy' )

    --[[  Set the Camera  ]]--

    ScenarioInfo.Camera = import('/lua/simcamera.lua').SimCamera('WorldCamera')


    --[[  Custom Initialization of Armies  ]]--

    LOG(" ")
    tblGroup = InitializeArmies()
    LOG(" ")

    --[[   Load, Transport and Unload the Attacker's ACU to the Drop Zone            ]]--
    --[[   The Camera is tracking ACU while being transported + CONTROLS are LOCKED  ]]--

    LOG(" ")
    if attackUBC != nil then
        fastPrintText("The ACU is being transported to its Drop Zone, get ready !")
        TransportACUtoDropZone( 'Air_Transport_1', attackUBC, 'DZ_P0', 'DZ_P1' )
    else
        LOG("MANIMAL:  DEBUG  Transport of attackUBC (nil) is not possible .")
    end
    LOG(" ")


    --[[  Spawn And Send Air Spies to Patrol around the Drop Zone  ]]--

    LOG(" ")
    SpawnAndSendAirSpiesPatrolDropZone( 'Air_Spies_1', 'DZ_P0_1', 'Drop_Zone_1_Patrol' )
    LOG(" ")


    --[[  Spawn and arm Tactical Missiles Launcher Silo for DEFENDER  ]]--

    LOG(" ")
    ArmerLanceMissilesTactique('Tactical_Missiles_1', 'TML_P0')
    LOG(" ")


    --[[  Warn the players when ALL Units are READY, and ACU in Drop Zone  ]]--

    WaitForACU()


end  # END of onPopulate






--[[                                                                             ]]--
--[[   OnStart(self)                                                             ]]--
--[[                                                                             ]]--
--[[   Game mechanisms are there                                                 ]]--
--[[                                                                             ]]--

function OnStart(self)

    LOG(" ")
    LOG("=======>  OnStart     ===============================================")
    LOG(" ")

    --[[  Show Enemy life bars  ]]--

    Utilities.UserConRequest("ui_ForceLifbarsOnEnemy")

    --[[  Check and Update a Table of the Units for an eventual tweaking  ]]--            

    CheckAliveUnits()

end






--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--
--[[                                                                             ]]--
--[[  OTHER FUNCTIONS                                                            ]]--
--[[                                                                             ]]--
--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--




--[[                                                                             ]]--
--[[   InitializeArmies()                                                        ]]--
--[[                                                                             ]]--
--[[   Spawn CUSTOM COMMANDERS and their ARMIES                                  ]]--
--[[                                                                             ]]--

function InitializeArmies()

    local tblGroups = {}
    local tblArmy = ListArmies()

    for iArmy, strArmy in pairs(tblArmy) do

        local tblData = Scenario.Armies[strArmy]

        if tblData then
		
            --[[  Spawn Upgraded ACUs  ]]--

            local cdrUnit = false
			
            LOG(" ")
            cdrUnit = SpawnUpgradedUBC( strArmy, iArmy ) 

            if cdrUnit != nil then

                LOG("MANIMAL:  SPAWN  Enhanced ACU for player "..iArmy.." ".. strArmy.." COMPLETED !")

                if strArmy == ubcsTable[theAttacker].name then
                    attackUBC = cdrUnit
                elseif strArmy == ubcsTable[theDefender].name then
                    defensUBC = cdrUnit
                end

                CustomizePlayersName( cdrUnit, iArmy )

            end
            LOG(" ")

            DoBuildRestrictions( iArmy, strArmy )
            LOG(" ")

            --[[  Pre-Built UNITS/BUILDINGS  ]]--            

            PreBuiltBase =  ScenarioUtils.CreateArmyGroup(strArmy, 'INITIAL')

            if PreBuiltBase != nil then
                LOG(" ")
                LOG("MANIMAL:  SPAWN de PREBUILT BASE for player ".. iArmy .. " " .. strArmy .. " .")
                LOG(" ")
            end

            SetArmyEconomy( strArmy, tblData.Economy.mass, tblData.Economy.energy)

        end   #end IF tblData

    end   #end FOR 


    return tblGroups

end




--[[                                                                             ]]--
--[[   SpawnUpgradedUBC( strArmy, iArmy )                                        ]]--
--[[                                                                             ]]--
--[[   Spawn an Upgraded ACU                                                     ]]--
--[[                                                                             ]]--

function SpawnUpgradedUBC( strArmy, iArmy )

    local cdrUnit = false

    local ubcId         = ubcsTable[iArmy].unitId             # current ACU Identifier
    local ubcName       = ubcsTable[iArmy].name               # current ACU name
    local ubcIdAttacker = ubcsTable[theAttacker].unitId       # Attacker s ACU Identifier
    local ubcIdDefender = ubcsTable[theDefender].unitId       # Defenders ACU Identifier

    if (ubcId == ubcIdDefender) or (ubcId == ubcIdAttacker) then  # ACU Identifier is valid

        --[[  spawn uef commander  ]]--

        local markerName = strArmy
        cdrUnit = SpawnUnit( ubcName, ubcsTable, markerName )

        --[[  successful spawn  ]]--

        if cdrUnit != nil then

            DoEnhancements( cdrUnit, iArmy, strArmy )

            DoRestrictEnhancements( iArmy, strArmy )

            --[[  anihilation type victory condition  ]]--

            cdrUnit.OldOnKilled = cdrUnit.OnKilled

            --[[  action when killed  ]]--

            cdrUnit.OnKilled = function( self, instigator, type, overkillRatio )
                GetArmyBrain( strArmy ):OnDefeat()
                GetArmyBrain( strEnemi(iArmy) ):OnVictory()
                cdrUnit.OldOnKilled( self, instigator, type, overkillRatio )	
            end

        else
            LOG("MANIMAL:  DEBUG  Unit Creation (ACU) (nil) not possible for player ".. iArmy .. " " .. strArmy .. " .")
        end   #end IF cdrUnit

   end   #end IF ubcId

   return cdrUnit

end




--[[                                                                             ]]--
--[[  DoEnhancements( unit, iArmy, strArmy )                                     ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--

function DoEnhancements( unit, iArmy, strArmy )

    if ubcsTable[iArmy].EnhancementsList then 
        for i,enhancement in ubcsTable[iArmy].EnhancementsList do
            unit:CreateEnhancement(enhancement)
            LOG("MANIMAL:  ACU Enhancement at index=" .. i .. ", enhancement=" .. enhancement .. " for player ".. iArmy .. " " .. strArmy .. " .")
        end
    end

end




--[[                                                                             ]]--
--[[  DoRestrictEnhancements( iArmy, strArmy )                                   ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--

function DoRestrictEnhancements( iArmy, strArmy )

    if ubcsTable[iArmy].EnhRestrictionsList then 
        for i,restriction in ubcsTable[iArmy].EnhRestrictionsList do
            ScenarioFramework.RestrictEnhancements(restriction)
            LOG("MANIMAL:  ACU RestrictEnhancement at index=" .. i .. ", restriction=" .. restriction .. " for player ".. iArmy .. " " .. strArmy .. " .")
        end
    end

end




--[[                                                                             ]]--
--[[  DoBuildRestrictions( iArmy, strArmy )                                      ]]--
--[[                                                                             ]]--
--[[  Take away units that the players shouldn't have access to                  ]]--
--[[                                                                             ]]--

function DoBuildRestrictions( iArmy, strArmy )

    if ubcsTable[iArmy].BuildRestrictionsList then 
        for i,catRestrict in ubcsTable[iArmy].BuildRestrictionsList do
            ScenarioFramework.AddRestriction( strArmy, catRestrict )
            LOG("MANIMAL:  Add Build Restriction at index=" .. i .. " for player ".. iArmy .. " " .. strArmy .. " .")
        end
    end


end




--[[                                                                             ]]--
--[[  CustomizePlayersName( unit, iArmy )                                        ]]--
--[[                                                                             ]]--
--[[  customize players's name as explicitly as possible                         ]]--
--[[  it's reset for some unknown reason when AUC is transported !               ]]--
--[[                                                                             ]]--

function CustomizePlayersName( unit, iArmy )

    ArmyBrains[iArmy].Nickname = ArmyBrains[iArmy].Nickname .. ubcsTable[iArmy].nickname
    ubcsTable[iArmy].nickname = ArmyBrains[iArmy].Nickname
    unit:SetCustomName( ArmyBrains[iArmy].Nickname )

end




--[[                                                                             ]]--
--[[  FixUpCustomizedPlayersName( attackACU, defensACU )                         ]]--
--[[                                                                             ]]--
--[[  Fixes customized name                                                      ]]--
--[[  it's reset for some unknown reason when AUC is transported !               ]]--
--[[  customize players's name as explicitly as possible                         ]]--
--[[                                                                             ]]--

function FixUpCustomizedPlayersName( attackACU, defensACU )

    LOG("MANIMAL:  FIX UP ArmyBrains[theAttacker].Nickname = "..ArmyBrains[theAttacker].Nickname)
    attackACU:SetCustomName( ArmyBrains[theAttacker].Nickname )
    LOG("MANIMAL:  FIX UP ArmyBrains[theDefender].Nickname = "..ArmyBrains[theDefender].Nickname)
    defensACU:SetCustomName( ArmyBrains[theDefender].Nickname )
    fastPrintText("F I G H T  !")

end




--[[                                                                             ]]--
--[[   TransportACUtoDropZone( transpName, cdrUnit, markerName, toDestin )       ]]--
--[[                                                                             ]]--
--[[   Load, Transport and Unload the Attacker's ACU to the Drop Zone            ]]--
--[[   The Camera is tracking it while transportinng + CONTROLS are LOCKED       ]]--
--[[                                                                             ]]--

function TransportACUtoDropZone( transpName, cdrUnit, markerName, toDestin )

    --[[  Spawn one Air Transport for Attacker ACU at spawnLoc ]]--
    A_Air_Transport_1 = SpawnUnit( transpName, unitsTable, markerName )

    --[[  Successful ACU spawn so the transport can be loaded up !  ]]--

    if A_Air_Transport_1 != nil then

        if not cdrUnit:IsDead() then
            --[[  Automatically attaches <units> to attach points on <transports>  ]]--

            ScenarioFramework.AttachUnitsToTransports( { cdrUnit }, { A_Air_Transport_1 } )
        end

        --[[  UNLOADING ATTACKER FROM AIR TRANSPORT  ]]--
        --[[  CONTROLS are LOCKED until ACU is in Drop Zone   ]]--
 
        if not cdrUnit:IsDead() and cdrUnit:IsUnitState('Attached') then

            --[[  Camera Tracking the Transport with ACU onboard  ]]--

            if not A_Air_Transport_1:IsDead() then 
                local unlock = true
                LOG("MANIMAL:  Camera Tracking Entity "..transpName.." with ACU Attached to it .")
                CameraTrackEntity( cdrUnit, 34, unlock )
            end

            --[[  IssueTransportUnload( transports, landingLocation )  ]]--

            local destMarkerPos = { }

            if  toDestin != '' then
                destMarkerPos  = ScenarioUtils.MarkerToPosition( toDestin )
                IssueTransportUnload( { A_Air_Transport_1 }, destMarkerPos )
            else
                LOG("MANIMAL:  DEBUG  TransportACUtoDropZone unknown destination ! .")
            end

            local unitsAreStillAttached =  WaitForTransportUnload( { cdrUnit } , { A_Air_Transport_1 } )

            if not unitsAreStillAttached then
                LOG("MANIMAL:  Transport Unloaded at "..toDestin.." .")
            else
                LOG("MANIMAL:  DEBUG  TransportACUtoDropZone unitsAreStillAttached NOT TOGGLED ! .")
            end

        end

    end

end




--[[                                                                             ]]--
--[[   WaitForTransportUnload( units, transports )                               ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--

function WaitForTransportUnload( units, transports )

    local unitsAreStillAttached = true

    ForkThread ( function(self)

        while unitsAreStillAttached do
            unitsAreStillAttached = WaitForTransportUnloadThread( units , transports )
        end

        --[[  ACUisReady TRIGGER is waiting for the Transport to be Unloaded    ]]--
        --[[  When ACU is Unloaded, ACUisReady state is toggled                 ]]--

        if not unitsAreStillAttached then
            local unit     = units[1]
            local idxArmee = unit:GetArmy()
            ubcsTable[idxArmee].isReady = true
            ACUisReady = true
        else
            LOG("MANIMAL:  DEBUG  WaitForTransportUnload ACU UNLOAD NOT TOGGLED ! .")
        end

        KillThread(self)

        return unitsAreStillAttached

    end)

end




--[[                                                                             ]]--
--[[   WaitForTransportUnloadThread( units, transports )                         ]]--
--[[                                                                             ]]--
--[[   inspired from ScenarioFramework.lua                                       ]]--
--[[   function PlatoonAttackWithTransportsThread                                ]]--
--[[                   ( platoon, landingChain, attackChain, instant )           ]]--
--[[                                                                             ]]--

function WaitForTransportUnloadThread( units, transports )

    local attached = true

    while attached do

        WaitSeconds(3)

        local allDead = true

        for k,v in transports do
            if not v:IsDead() then
                allDead = false
                break
            end
        end

        if allDead then
           return
        end

        attached = false

        for num, unit in units do
            if not unit:IsDead() and unit:IsUnitState('Attached') then
                attached = true
                break
            end
        end

    end

    return attached

end




--[[                                                                             ]]--
--[[   CameraTrackEntity( unit, seconds, unlock )                                ]]--
--[[                                                                             ]]--
--[[   see CameraTrackEntityThread for details on parameters                     ]]--
--[[                                                                             ]]--

function CameraTrackEntity( unit, seconds, unlock )
    ForkThread( CameraTrackEntityThread, unit, seconds, unlock )
end




--[[                                                                             ]]--
--[[   CameraTrackEntityThread( unit, seconds, unlock )                          ]]--
--[[   - unit is the unit to be tracked by the camera                            ]]--
--[[   - seconds is the duration of the tracking                                 ]]--
--[[   - unlock if true, unlocks the controls                                    ]]--
--[[                                                                             ]]--
--[[   Camera will follow the specified unit for a given duration in seconds     ]]--
--[[   CONTROLS are LOCKED for this duration                                     ]]--
--[[   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                     ]]--
--[[   Inspired from the function OperationCamera                                ]]--
--[[   d�rivable en CameraTrackEntities( { unit1, unit2, ... }, unlock )         ]]--
--[[                                                                             ]]--

function CameraTrackEntityThread( unit, seconds, unlock )

    local Tracking = import('/lua/ui/game/tracking.lua')
    local camera   = ScenarioInfo.Camera
    local heading = 0
    local pos = false
    local zoomVar = 50
    local marker   = false

    LockInput()

    camera:UseSystemClock()
    WaitTicks(1)

    if unit then

        heading = unit:GetHeading()
        pos     = unit:GetPosition()
        marker  = {
            orientation = VECTOR3( heading, 0, 0 ),
            position = { pos[1], pos[2], pos[3] },
            zoom = zoomVar,
        }

        camera:TrackEntity( unit, zoomVar, seconds )

    end

    if (unlock) then

        if ( seconds ) and ( seconds != 0 ) then
            WaitForCamera()
        end

        UnlockInput()

    end

end




--[[                                                                             ]]--
--[[   WaitForCamera()                                                           ]]--
--[[                                                                             ]]--
--[[   Used by other functions to make sure that they don't return               ]]--
--[[   before the camera is done moving.                                         ]]--
--[[                                                                             ]]--
--[[   Inspired from the cinematics.lua functions                                ]]--
--[[                                                                             ]]--

function WaitForCamera()

    --[[  Wait for it to be done  ]]--

    ScenarioInfo.Camera:WaitFor()

    --[[  Reset the event tracker, so that the next  ]]--
    --[[  camera action can be waited for            ]]--

    ScenarioInfo.Camera:EventReset()

end




--[[                                                                             ]]--
--[[  WaitForACU()                                                               ]]--
--[[                                                                             ]]--
--[[  Warn the players when ALL Units are READY, and ACU in Drop Zone            ]]--
--[[                                                                             ]]--

function WaitForACU()

    ForkThread( function (self) 

        local go = false
  
        go = WaitForACUThread()
 
        if go then
            UnlockInput()
        end

        KillThread(self)

    end)

end




--[[                                                                             ]]--
--[[  WaitForACUThread()                                                         ]]--
--[[                                                                             ]]--
--[[  Warn the players when ALL Units are READY, and ACU in Drop Zone            ]]--
--[[                                                                             ]]--

function WaitForACUThread()

    while not ACUisReady do
        WaitSeconds(1)

        if ACUisReady then
            break
        end
    end

    LOG(" ")
    if ACUisReady then
        FixUpCustomizedPlayersName( attackUBC, defensUBC )
    else
        LOG("MANIMAL:  DEBUG  ForkThread ACUisReady => FALSE (Locked!) .")
    end
    LOG(" ")

   return ACUisReady

end




--[[                                                                             ]]--
--[[  SpawnAndSendAirSpiesPatrolDropZone( unitName, markerName, patrolChain )    ]]--
--[[                                                                             ]]--
--[[  Spawn And Send Air Spies to Patrol around the Drop Zone                    ]]--
--[[                                                                             ]]--

function SpawnAndSendAirSpiesPatrolDropZone( unitName, markerName, patrolChain )

    SpawnAirSpies( unitName, markerName )

    --[[  successful spawn so the Spies can go to Patrol !      ]]--
    --[[  assumes the total of spawned units matches numUnit    ]]--

    if A_Air_Spies_1 != nil then 
        --[[  Make AirSpies patrol around the Drop Zone  ]]--
        SendAirSpiesPatrolDropZone( A_Air_Spies_1, patrolChain )
    end

end




--[[                                                                             ]]--
--[[  SpawnAirSpies( unitName, markerName )                                      ]]--
--[[                                                                             ]]--
--[[  Spawn Air Spies                                                            ]]--
--[[                                                                             ]]--

function SpawnAirSpies( unitName, markerName )

    --[[  Verify the GROUP A_Air_Spies_1 exists  ]]--

    if A_Air_Spies_1 != nil then  
  
        local numUnit = table.getsize(A_Air_Spies_1)
        local index = UnitNameToIndex(unitName, unitsTable)

        if index > 0 then 
            --[[  Spawn a GROUP of 10 SpyPlanes (Cybran Tech 2)  ]]--

            for unitCount=1, numUnit do

                local Air_Spy = A_Air_Spies_1[unitCount]

                if not Air_Spy then

                    Air_Spy = SpawnUnit( unitName, unitsTable, markerName )

                    if Air_Spy != nil then

                        A_Air_Spies_1[unitCount] = Air_Spy
                        LOG("MANIMAL:  Air_Spies_1 n� "..unitCount.." is READY for "..unitsTable[index].army.." at "..markerName.."  .")
                    end
                end
            end

        else
            LOG("MANIMAL:  DEBUG  Spawn  NO Air_Spies (index NUL) ! .")
        end
    else
        LOG("MANIMAL:  DEBUG  Spawn  A_Air_Spies NIL ! .")
    end

end




--[[                                                                             ]]--
--[[  SendAirSpiesPatrolDropZone( units, patrolChain )                           ]]--
--[[                                                                             ]]--
--[[  Send Air Spies to Patrol around the Drop Zone                              ]]--
--[[                                                                             ]]--

function SendAirSpiesPatrolDropZone( units, patrolChain )

    local attackLocs = ScenarioUtils.ChainToPositions(patrolChain)

    --[[  Make AirSpies patrol around the Drop Zone  ]]--

    for k,v in attackLocs do

        IssuePatrol( units, v )

        LOG("MANIMAL:  A_Air_Spies_1 PATROLLING Drop_Zone_1_Patrol, patrol point n�"..k.." .")
    end

end




--[[                                                                             ]]--
--[[  ArmerLanceMissilesTactique( unitName, markerName )                         ]]--
--[[                                                                             ]]--
--[[  Spawn and arm Tactical Missiles Launcher Silo for DEFENDER                 ]]--
--[[                                                                             ]]--

function ArmerLanceMissilesTactique( unitName, markerName )

    --[[  Spawn Tactical Missiles Launcher for Defender ACU  ]]--

    D_Tactical_Missiles_1 = SpawnUnit( unitName, unitsTable, markerName )

    --[[  successful spawn so the Silo can be filled up of Ammo !  ]]--

    if D_Tactical_Missiles_1 != nil then

        --[[  BUILDING TACTICAL MISSILES (FULL SILO)  ]]--

        local NumToBuild = 6
        D_Tactical_Missiles_1:GiveTacticalSiloAmmo(NumToBuild)
        LOG("MANIMAL:   Tactical Missiles Silo Ammo = " .. NumToBuild .. " at "..markerName.."  .")

    end

end




--[[                                                                             ]]--
--[[   SpawnUnit( unitName, unitsTbl, markerName )                               ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--

function SpawnUnit( unitName, unitsTbl, markerName )

    local spawnedUnit = false
    local spawnLoc    = { false, false, false }

    local index = UnitNameToIndex( unitName, unitsTbl )

    if index > 0 then

        local unit  = unitsTbl[index]

        spawnLoc = mPos( markerName )

        if (unit != nil) and (unit.name == unitName) then

            spawnedUnit = CreateUnitHPR( unit.unitId, unit.army, spawnLoc[1], spawnLoc[2], spawnLoc[3], 0, 0, 0 )

            LOG("MANIMAL:  SPAWN  unit = "..unit.name.." (Id = "..unit.unitId..")  for Army = "..unit.army.."  at marker = "..markerName.."  .")

        else
            LOG("MANIMAL:  DEBUG  UNABLE to SPAWN  unit (index NUL) !  .")
        end
    else
        LOG("MANIMAL:  DEBUG  UNABLE to SPAWN  unit (NIL) !  .")
    end

    return spawnedUnit

end




--[[                                                                             ]]--
--[[   CheckAliveUnits()                                                         ]]--
--[[                                                                             ]]--
--[[   Check the Defense Units that are currently present ("alive") on the Map   ]]--
--[[   The Table of the built Defense Units is used for tweaking these ones      ]]--            
--[[                                                                             ]]--
--[[   Wait a minimum of 1 second between 2 checks, that is shortest build time  ]]--
--[[                                                                             ]]--

function CheckAliveUnits()

    ForkThread ( function(self)

        while not attackUBC:IsDead() and not defensUBC:IsDead() do

            --[[  Wait a minimum of 1 second between 2 checks NOT LESS !  ]]--
            WaitSeconds(1)

            if ACUisReady then
                CheckAliveUnitsThread()
            end

        end

        KillThread(self)
    end)

end




--[[                                                                             ]]--
--[[   CheckAliveUnitsThread()                                                   ]]--
--[[                                                                             ]]--
--[[   Check ALL Units that are currently present ("alive") on the Map           ]]--
--[[                                                                             ]]--

function CheckAliveUnitsThread()

    local actualUnits = GetUnitsInRect(Rect(0,0,mapSizeX,mapSizeY))

    if actualUnits != nil then

        --[[  Process all of the current units found  ]]--

        for i, unit in actualUnits do

            if unit != nil then

                --[[  Check 1 unit and do eventual Specification Modification on it !  ]]--

                CheckSingleAliveUnit( unit )

            else
                LOG("MANIMAL:  DEBUG  CheckAliveUnitsThread  unit NIL !  .")
            end

        end  ## end FOR unit in actualUnits

    else
        LOG("MANIMAL:  DEBUG  CheckDefenseUnitsThread  actualUnits NIL !  .")
    end

end




--[[                                                                             ]]--
--[[   CheckSingleAliveUnit( unit )                                              ]]--
--[[                                                                             ]]--
--[[   Check a single unit that's currently present ("alive") on the Map         ]]--
--[[                                                                             ]]--

function CheckSingleAliveUnit( unit )

    local bp       = ( unit:GetBlueprint() or false )
    local BPidStr  = ""

    --[[  Modifier et Sauvegarder Economie de l'unit� "vivante" courante  ]]--

    if bp != nil then 

        BPidStr = ExtractBpUnitIdStr( bp )  ## NAME of current unit ex:"urb2101"

        --[[  Current Unit is MODIFIABLE !  ]]--

        if UnitSpecIsToBeModified( BPidStr, unitsTable ) then          

            --[[  Current Unit doesn't exist in AliveUnits  ]]--

            if not UnitSyncIdStrIsInTable( unit.Sync.id, aliveUnits ) then


                if not unit:IsDead() and not UnitSyncIdStrIsInTable( unit.Sync.id, aliveUnits ) then

                    local info = "New Unit ("..unit.Sync.id..")" 
                    IncreaseUnitBuildTime( info, unit )
                    unit:RequestRefreshUI()
                end


                --[[  Customized OnReclaimed Method  ]]--

                unit.OldOnReclaimed = unit.OnReclaimed

                unit.OnReclaimed = function( self, entity )

                    local info = "Unit Reclaimed ("..unit.Sync.id..")"
                    DecreaseUnitBuildTime( info, unit )
                    unit:RequestRefreshUI()

                    unit.OldOnReclaimed( self, entity )
                end


                --[[  Customized OnKilled Method  ]]--

                unit.OldOnKilled = unit.OnKilled

                unit.OnKilled = function( self, instigator, type, overkillRatio )

                    local info = "Unit Killed ("..unit.Sync.id..")"
                    DecreaseUnitBuildTime( info, unit )
                    unit:RequestRefreshUI()

                    unit.OldOnKilled( self, instigator, type, overkillRatio )	
                end


                --[[  Customized OnCaptured Method  ]]--

                unit.OldOnCaptured = unit.OnCaptured

                unit.OnCaptured = function( self, captor )

                    local captorArmyIndex = captor:GetArmy()

                    local info = "Unit Captured ("..unit.Sync.id..") by army "..captorArmyIndex
                    --[[  Considered as KILLED for ex-owner, in NEXT ITERATION as a NEW UNIT for CAPTOR !  ]]--
                    DecreaseUnitBuildTime( info, unit )
                    unit:RequestRefreshUI()

                    unit.OldOnCaptured( self, captor )
                end

            end 

        end

    else
        LOG("MANIMAL:  DEBUG  CheckSingleAliveUnit  blueprint NIL !  .")
    end

end




--[[                                                                             ]]--
--[[   IncreaseUnitBuildTime( info, unit )                                       ]]--
--[[                                                                             ]]--
--[[   - info is a text info message to be displayed                             ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--

function IncreaseUnitBuildTime( info, unit )

    local uSpec = {}
    local bp       = ( unit:GetBlueprint() or false )
    local BPidStr  = ( ExtractBpUnitIdStr( bp ) or "" )
    local uSync    = ( unit.Sync or false )

    CalculateNewBuildTime( info, BPidStr, bp )

    uSpec.numbIdStr = uSync.id    ## NUMBER of current unit ex: unit.Sync.id = "1457"
    uSpec.nameIdStr = BPidStr
    uSpec.army      = uSync.army

    StoreValuesInAliveUnitsTable( uSpec, aliveUnits )

end




--[[                                                                             ]]--
--[[   CalculateNewBuildTime( info, BPidStr, bp )                                ]]--
--[[                                                                             ]]--
--[[   - info is a text info message to be displayed                             ]]--
--[[   - bp is  the unit's blueprint                                             ]]--
--[[                                                                             ]]--
--[[   Calculate the NEW value of BuildTime Multiplier for current unit          ]]--
--[[                                                                             ]]--

function CalculateNewBuildTime( info, BPidStr, bp )

    local bpEcon  = ( bp.Economy or false )
    local uIndex  = ( BPHelpTextToIndex( BPidStr, unitsTable ) or 0 )

    local spexIdx = BPidStrToIndex( BPidStr, BuildTimeTable )

    --[[  current unit's BuildTime Multiplier is to be altered ( BTM=1, no changes) ]]--

    if unitsTable[uIndex].BuildTimeMultiplier != 0 then   

        # set BuildTime on default value whenever the result is false
        local newBuildTime = ( BuildTimeTable[spexIdx].Current * unitsTable[uIndex].BuildTimeMultiplier ) or BuildTimeTable[spexIdx].Original

        BuildTimeTable[spexIdx].Current  = MATH_IRound( tonumber( newBuildTime ) )  # keep INTEGRAL part ONLY

        bpEcon.BuildTime = BuildTimeTable[spexIdx].Current

LOG("MANIMAL:  DEBUG  "..info.." ! name = "..BPidStr.." - BT ="..BuildTimeTable[spexIdx].Current.."  Orig="..BuildTimeTable[spexIdx].Original.."  spexIdx="..spexIdx.."  .")

    end

end




--[[                                                                             ]]--
--[[   StoreValuesInAliveUnitsTable( unitSpec, aliveUnitsTbl )                   ]]--
--[[                                                                             ]]--

function StoreValuesInAliveUnitsTable( unitSpec, aliveUnitsTbl )
    table.insert( aliveUnitsTbl, unitSpec )
end




--[[                                                                             ]]--
--[[   DecreaseUnitBuildTime( info, unit )                                      ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--

function DecreaseUnitBuildTime( info, unit )

    local uSpec    = {}
    local bp       = ( unit:GetBlueprint() or false )
    local BPidStr  = ( ExtractBpUnitIdStr( bp ) or "" )
    local uSync    = ( unit.Sync or false )

    RestorePreviousBuildTime( info, BPidStr, bp )

    uSpec.numbIdStr = uSync.id    ## NUMBER of current unit ex: unit.Sync.id = "1457"
    uSpec.nameIdStr = BPidStr
    uSpec.army      = uSync.army

    RemoveUnitFromAliveUnitsTable( uSpec, aliveUnits )

end




--[[                                                                             ]]--
--[[   RestorePreviousBuildTime( info, BPidStr, bp )                             ]]--
--[[                                                                             ]]--
--[[   info is a text info message to be displayed                               ]]--
--[[                                                                             ]]--
--[[   Restore the PREVIOUS value of BuildTime Multiplier for current unit       ]]--
--[[                                                                             ]]--

function RestorePreviousBuildTime( info, BPidStr, bp )

    local bpEcon  = ( bp.Economy or false )
    local uIndex  = ( BPHelpTextToIndex( BPidStr, unitsTable ) or 0 )

    local spexIdx = BPidStrToIndex( BPidStr, BuildTimeTable )

    --[[  current unit's BuildTime Multiplier is to be altered ( BTM=1, no changes) ]]--

    if unitsTable[uIndex].BuildTimeMultiplier != 0 then

        # set BuildTime on default value whenever the result is false
        local prevBuildTime = ( BuildTimeTable[spexIdx].Current / unitsTable[uIndex].BuildTimeMultiplier ) or BuildTimeTable[spexIdx].Original

        if prevBuildTime >= BuildTimeTable[spexIdx].Original then

            BuildTimeTable[spexIdx].Current = MATH_IRound( tonumber( prevBuildTime ) )  # keep INTEGRAL part ONLY
        else

            BuildTimeTable[spexIdx].Current = BuildTimeTable[spexIdx].Original
        end

        bpEcon.BuildTime = BuildTimeTable[spexIdx].Current

LOG("MANIMAL:  DEBUG  "..info.." ! name = "..BPidStr.." - BT ="..BuildTimeTable[spexIdx].Current.."  Orig="..BuildTimeTable[spexIdx].Original.."  spexIdx="..spexIdx.."  .")

    end

end




--[[                                                                             ]]--
--[[   RemoveUnitFromAliveUnitsTable( unitSpec, aliveUnitsTbl )                  ]]--
--[[                                                                             ]]--
--[[   unit is a "dead" unit to be removed from table                            ]]--
--[[   aliveUnitsTbl is a table of the current alive units                       ]]--
--[[                                                                             ]]--
--[[   Remove the current unit from alive units table                            ]]--
--[[                                                                             ]]--

function RemoveUnitFromAliveUnitsTable( unitSpec, aliveUnitsTbl )

    local unitIsRemoved = false

    for i, aliveUnit in pairs( aliveUnitsTbl ) do

        if aliveUnit.numbIdStr == unitSpec.numbIdStr then

            local numAliveUnit = i

            if numAliveUnit > 0 then

                table.remove( aliveUnitsTbl, numAliveUnit )
                unitIsRemoved = true
            end

            break
        end

    end

    return unitIsRemoved

end







--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   UTILITY FUNCTIONS                                                         ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--




--[[                                                                             ]]--
--[[   UnitSpecIsToBeModified( blueprintIdStr, unitsTbl )                        ]]--
--[[   - unitsTbl is the table of modifiable units                               ]]--
--[[   - blueprintIdStr contains a part of the HelpText field (string)           ]]--
--[[                                                                             ]]--
--[[   Returns TRUE if the unit.sync.id matches the current unit's blueprint     ]]--
--[[   HelpText field  AND if its specifications are to be changed which is      ]]--
--[[   done by checking if  Comsumption and Modifier are < or == to the          ]]--
--[[   ubcsTable's matching values in order to not point already altered units   ]]--
--[[                                                                             ]]--

function UnitSpecIsToBeModified( blueprintIdStr, unitsTbl )

    local isToBeModified = false
    local index          = 0

    if ( blueprintIdStr != '' ) then

        index = UnitIdStrToIndex( blueprintIdStr, unitsTbl )

        --[[  index > 0 if blueprintIdStr exists in unitsTbl  ]]--

        if index > 0 then

            if unitsTbl[index].BuildTimeMultiplier > 1  then

                isToBeModified = true
            end
        end
    else
        LOG("MANIMAL:  DEBUG  UnitSpecIsToBeModified  blueprintIdStr MISSING ! .")
    end

    return isToBeModified

end




--[[                                                                             ]]--
--[[   ExtractBpUnitIdStr( blueprint )                                           ]]--
--[[                                                                             ]]--
--[[   Copy "uxxnnnn" from "<LOC uxxnnnn_help>aaaaaaaa" (characters 6 to 12)     ]]--
--[[                                                                             ]]--

function ExtractBpUnitIdStr( blueprint )

    --[[  string.sub (s, i [, j]) from LUA  Guide  ]]--

    return ( string.sub( blueprint.Interface.HelpText, 6, 12 )  or "" )

end




--[[                                                                             ]]--
--[[   UnitSyncIdStrIsInTable( unitSyncId, aliveUnitsTbl )                       ]]--
--[[                                                                             ]]--
--[[   unitSyncId contains a SYNC Identifier string                              ]]--
--[[   example: unit.Sync.id="1048611"                                           ]]--
--[[   aliveUnitsTbl is a table of the current alive units                       ]]--
--[[   first field of a unit's spec is: unitSpec.numbIdStr                       ]]--
--[[                                                                             ]]--
--[[   Returns TRUE if the current unit is already within the aliveUnitsTbl      ]]--
--[[                                                                             ]]--

function UnitSyncIdStrIsInTable( unitSyncId, aliveUnitsTbl )

    local unitSyncIdIsInTbl = false

    for i, aliveUnit in pairs( aliveUnitsTbl ) do

        if aliveUnit.numbIdStr == unitSyncId then 

            unitSyncIdIsInTbl = true
            break
        end

    end

    return unitSyncIdIsInTbl

end




--[[                                                                             ]]--
--[[   BPidStrToIndex( blueprintIdStr, buildTimeTbl )                            ]]--
--[[                                                                             ]]--
--[[   - blueprintIdStr is units help text descriptor                            ]]--
--[[   - buildTimeTbl is a table of BuildTime values                             ]]--
--[[                                                                             ]]--
--[[   Returns the Index of the unit which has Indent matching BluePrint         ]]--
--[[                                                                             ]]--

function BPidStrToIndex( blueprintIdStr, buildTimeTbl )

    local spexIdx = 0

    for i, uSpex in pairs( buildTimeTbl ) do

        if uSpex.idStr == blueprintIdStr then

            spexIdx = i
            break
        end
    end

    return spexIdx
end




--[[                                                                             ]]--
--[[   BPHelpTextToIndex( blueprintIdStr, unitsTbl )                             ]]--
--[[                                                                             ]]--
--[[   - blueprintIdStr is units help text descriptor                            ]]--
--[[   - unitsTable is a table of units descriptors                              ]]--
--[[                                                                             ]]--
--[[   Returns the Index of the unit which has Indent matching BluePrint         ]]--
--[[                                                                             ]]--

function BPHelpTextToIndex( blueprintIdStr, unitsTbl )

    local index = 0

    if blueprintIdStr != '' then

        index = UnitIdStrToIndex( blueprintIdStr, unitsTbl )

    else
        LOG("MANIMAL:  DEBUG  BPHelpTextToIndex  blueprintIdStr=EMPTY ! .")
    end

    return index

end




--[[                                                                             ]]--
--[[   UnitIdStrToIndex( unitId, unitsTbl )                                      ]]--
--[[                                                                             ]]--
--[[   - unitId is the unit identifier string (example 'ueb2101')                ]]--
--[[   - unitsTable is a table of units descriptors                              ]]--
--[[                                                                             ]]--
--[[   Returns the Index of the unit with matching UnitId in unitsTbl            ]]--
--[[                                                                             ]]--

function UnitIdStrToIndex( unitId, unitsTbl )

    local index = 0

    for i, unit in pairs( unitsTbl ) do
        if unit.unitId == unitId then 
            index = i
            break
        end
    end

    return index

end




--[[                                                                             ]]--
--[[   UnitNameToIndex( unitName, unitsTbl )                                     ]]--
--[[                                                                             ]]--
--[[   unitsTable is a table of units descriptors                                ]]--
--[[                                                                             ]]--
--[[   Returns the Index of the unit with matching UnitName in unitsTbl          ]]--
--[[                                                                             ]]--

function UnitNameToIndex( unitName, unitsTbl )

    local index = 0

    for i, unit in pairs( unitsTbl ) do
        if unit.name == unitName then 
            index = i
            break
        end
    end

    return index

end




--[[                                                                             ]]--
--[[   GetMarker( name )                                                         ]]--
--[[                                                                             ]]--
--[[   This is a copy of the function that's in ScenarioFramework                ]]--
--[[                                                                             ]]--
--[[   get marker given a name from map_save done in map editor                  ]]--
--[[   you can the use marker.position to get a location in the {x,y,z} format   ]]--
--[[                                                                             ]]--

function GetMarker( name )

    return Scenario.MasterChain._MASTERCHAIN_.Markers[name]

end




--[[                                                                             ]]--
--[[   mPos( name )                                                              ]]--
--[[                                                                             ]]--
--[[   name is the existing name of the marker                                   ]]--
--[[                                                                             ]]--
--[[   Get Marker position  { x, y, z }                                          ]]--
--[[                                                                             ]]--

function mPos( name )

    return GetMarker(name).position

end




--[[                                                                             ]]--
--[[   strEnemi( iArmy )                                                         ]]--
--[[                                                                             ]]--
--[[   iArmy is the index of the current Army                                    ]]--
--[[                                                                             ]]--
--[[   Returns the enemi of the current Army                                     ]]--
--[[                                                                             ]]--

function strEnemi( iArmy )

    return ubcsTable[iArmy].enemi

end




--[[                                                                             ]]--
--[[  indexToArmy( armyIndex )                                                   ]]--
--[[                                                                             ]]--
--[[  given an army index returns an army                                        ]]--
--[[                                                                             ]]--

function indexToArmy( armyIndex )

    return ListArmies()[armyIndex]

end




--[[                                                                             ]]--
--[[  fastPrintText(text)                                                        ]]--
--[[                                                                             ]]--
--[[  Display Text ingame for all players                                        ]]--
--[[                                                                             ]]--

function fastPrintText(text)

    ForkThread( function()

        PrintText( text, 20, nil, 5, 'center')

    end)

end




--[[  E N D  O F  S C R I P T  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ ]]--

--[[  ####################################     N O T E S     #################################### ]]--
--[[                                                                                              ]]--


